/*********************************************************************************

Copyright(c) 2014 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************/

#if defined(__ADSPSC584__)

/*
 * Use the power service to change processor mode and frequency settings.
 */

#include <services/pwr/adi_pwr.h>
#include <stdio.h>
#include <cdefSC58x_rom.h>

/* ADI initialization header */
#include "adi_initialize.h"

#include "sc58x_init.h"

#if 1

static bool bError = false;

static int callbackEvents;

static void CheckResult(ADI_PWR_RESULT result, char *message);
static void setDefaultPower(uint32_t cgu);
// extern void ConfigSoftSwitches(void);

/*
 * Report error if result code is not success
 */
static void CheckResult(ADI_PWR_RESULT result, char *message)
{
	if (result != ADI_PWR_SUCCESS) {
		// printf("%s failed\n", message);
		bError = true;
	}
}

/*
 * Reset processor to default power settings
 */
static void setDefaultPower(uint32_t cgu)
{
	ADI_PWR_RESULT result;

	result = adi_pwr_SetPllControlRegister(cgu, ADI_PWR_PLL_CTL_DISABLE, 0);
	result = adi_pwr_SetPllControlRegister(cgu, ADI_PWR_PLL_CTL_EXIT_ACTIVE, 1);
	result = adi_pwr_SetClkControlRegister(cgu, ADI_PWR_CLK_CTL_DF, DF_DEFAULT);
	result = adi_pwr_SetClkControlRegister(cgu, ADI_PWR_CLK_CTL_MSEL, MSEL_DEFAULT);
	result = adi_pwr_SetClkDivideRegister(cgu, ADI_PWR_CLK_DIV_SYSSEL, SYSSEL_DEFAULT);
	result = adi_pwr_SetClkDivideRegister(cgu, ADI_PWR_CLK_DIV_S0SEL, S0SEL_DEFAULT);
	result = adi_pwr_SetClkDivideRegister(cgu, ADI_PWR_CLK_DIV_S1SEL, S1SEL_DEFAULT);
	result = adi_pwr_SetClkDivideRegister(cgu, ADI_PWR_CLK_DIV_CSEL, CSEL_DEFAULT);
	result = adi_pwr_SetClkDivideRegister(cgu, ADI_PWR_CLK_DIV_DSEL, DSEL_DEFAULT);
}

/*
 * Power event Callback function
 */
static void PWRCallback(void *pCBParam, uint32_t Event, void *pArg)
{
	uint32_t sclk;
	ADI_PWR_MODE  mode;

	switch ( (ADI_PWR_EVENT)Event )
	{
    case ADI_PWR_EVENT_FREQ_PRE_CHANGE:
    	sclk = *(uint32_t*)(pArg);
    	callbackEvents++;
    	break;
    case ADI_PWR_EVENT_FREQ_POST_CHANGE:
    	sclk = *(uint32_t*)(pArg);
    	callbackEvents++;
    	break;
    case ADI_PWR_EVENT_MODE_PRE_CHANGE:
    	mode = (ADI_PWR_MODE)pArg;
    	callbackEvents++;
    	break;
    case ADI_PWR_EVENT_MODE_POST_CHANGE:
    	mode = (ADI_PWR_MODE)pArg;
    	callbackEvents++;
    	break;
    default:
    	break;
	}
}

#endif

void initcode(ADI_ROM_BOOT_CONFIG* pBootStruct)
{
	uint32_t   fcclk = 0u;
	uint32_t   fsysclk = 0u;
	uint32_t   fsclk0 = 0u;
	uint32_t   fsclk1 = 0u;
	uint32_t   fdclk = 0u;

	ADI_PWR_RESULT result;
	ADI_PWR_MODE  mode;
	int expectedEvents = 0;
#if 1
	bError = false;
	callbackEvents = 0;

	//Set the DLLRST bit (bit 11 of the DMC_PHY_CTL0 register) before CGU Initialization. More details at http://www.spd.analog.com/issues/dsp_pte_dev/issuemgr.cgi?item=36-10-0001
	*pREG_DMC0_PHY_CTL0=0x800;

	result = adi_pwr_Init(CGU_DEV, CLKIN);
	CheckResult(result, "adi_pwr_Init");

	result = adi_pwr_InstallCallback(CGU_DEV, PWRCallback);
	CheckResult(result, "adi_pwr_InstallCallback");

#if defined(__ADSPGCC__)
	/* Set CGU clock select register to CLKO2/4 (ARM core) */
	result = adi_pwr_SetClkOutSelectRegister(CGU_DEV, ADI_PWR_CLKOUT_CLKO2);
	CheckResult(result, "adi_pwr_SetClkOutSelectRegister");
#else
	/* Set CGU clock select register to CLKO0/4 (Sharc core) */
	result = adi_pwr_SetClkOutSelectRegister(CGU_DEV, ADI_PWR_CLKOUT_CLKO0);
	CheckResult(result, "adi_pwr_SetClkOutSelectRegister");
#endif

#if defined(CGU_BEFORE_POWER)
	/* Reset processor to default power settings */
	setDefaultPower(CGU_DEV);
#endif
	/* get the power mode */
	result = adi_pwr_GetPowerMode(CGU_DEV, &mode);
	CheckResult(result, "adi_pwr_GetPowerMode");
	if (mode != ADI_PWR_MODE_FULL_ON)
	{
		/* set power mode to full on */
		result = adi_pwr_SetPowerMode(CGU_DEV, ADI_PWR_MODE_FULL_ON);
		CheckResult(result, "adi_pwr_SetPowerMode");
		expectedEvents += 2; /* pre and post event */

		/* get the power mode */
		result = adi_pwr_GetPowerMode(CGU_DEV, &mode);
		CheckResult(result, "adi_pwr_GetPowerMode");
		if (mode != ADI_PWR_MODE_FULL_ON)
		{
			bError = true;
		}
	}

	result = adi_pwr_GetCoreFreq(CGU_DEV, &fcclk);
	CheckResult(result, "adi_pwr_GetCoreFreq");

	result = adi_pwr_GetSystemFreq(CGU_DEV, &fsysclk, &fsclk0, &fsclk1);
	CheckResult(result, "adi_pwr_GetSystemFreq");

	result = adi_pwr_GetDDRClkFreq(CGU_DEV, &fdclk);
	CheckResult(result, "adi_pwr_GetDDRClkFreq");

	//printf("CCLK = %d, SYSCLK = %d, SCLK0 = %d, SCLK1 = %d, DCLK = %d\n", fcclk, fsysclk, fsclk0, fsclk1, fdclk);


	result = adi_pwr_RemoveCallback(CGU_DEV, PWRCallback);
	CheckResult(result, "adi_pwr_RemoveCallback");

	/* check num callback events */
	if (expectedEvents != callbackEvents) {
		bError = true;
		//printf("Wrong number of callback events\n");
	}
#if !defined(CGU_BEFORE_POWER)
	/* Reset processor to default power settings */
	setDefaultPower(CGU_DEV);
#endif


	//Clear the DLLRST bit (bit 11 of the DMC_PHY_CTL0 register) after CGU Initialization. More details at http://www.spd.analog.com/issues/dsp_pte_dev/issuemgr.cgi?item=36-10-0001
	*pREG_DMC0_PHY_CTL0=0x000;
#else
	Init_CGU();
#endif
	int i = 0;

	// http://www.spd.analog.com/issues/dsp_pte_dev/issuemgr.cgi?item=36-10-0002
	/////////////////////////////////////////////
	*pREG_DMC0_PHY_CTL2 |= 0xC000000;
	*pREG_DMC0_PHY_CTL0 &= ~0xF;
	*pREG_DMC0_PHY_CTL0 |= 0xF;

	*pREG_DMC0_PHY_CTL2 |= 0xC000000;
	*pREG_DMC0_PHY_CTL2 &= ~0xF;
	*pREG_DMC0_PHY_CTL2 |= 0xF;
	/////////////////////////////////////////////

	*pREG_DMC0_PHY_CTL1 = 0x00000000;
	*pREG_DMC0_PHY_CTL4 = 0x00000001;

	// Program the PAD RTT and driver impedence values required here
	*pREG_DMC0_CAL_PADCTL0 = 0xE0000000;
	*pREG_DMC0_CAL_PADCTL2 = 0x00782828;

	// Start calibration
	*pREG_DMC0_CAL_PADCTL0 |= 0x10000000;

	// Wait for PAD calibration to complete - 300 DCLK cycle. Worst case: CCLK=450 MHz, DCLK=125 MHz
	for(i=0; i<1200; i++);

	// Wait for DLL lock- 4500 DCLK cycles, Worst case: CCLK=400 MHz, DCLK=125 MHz
	for(i=0;i<18000;i++);

	*pREG_DMC0_CFG = 0x00000522;
	*pREG_DMC0_TR0 = 0x21610535;
	*pREG_DMC0_TR1 = 0x404e0c30;
	*pREG_DMC0_TR2 = 0x00326312;
	*pREG_DMC0_MR = 0x00000a52;
	*pREG_DMC0_EMR1 = 0x00000004;
	*pREG_DMC0_EMR2 = 0x00000000;

	*pREG_DMC0_CTL = 0x00000404;

	// Wait till INIDONE is set
	while((*pREG_DMC0_STAT&BITM_DMC_STAT_INITDONE)==0);

	*pREG_DMC0_DLLCTL = 0x00000948;

	int temp11, *ptemp11;
	ptemp11=0x80000000;	//Dummy read
	temp11=*ptemp11;
	*pREG_DMC0_PHY_CTL0|=0x1000;
	*pREG_DMC0_PHY_CTL0&=~0x1000;

	return;
}


#endif /* __ADSPSC584__ */
