/*********************************************************************************

Copyright(c) 2014 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************/

#if defined(__ADSPSC584__)

/*
 * Use the power service to change processor mode and frequency settings.
 */

#include <services/pwr/adi_pwr.h>
#include <stdio.h>
#include <cdefSC58x_rom.h>
#include "DMCInit.h"

/* ADI initialization header */
#include "adi_initialize.h"

#include "sc58x_init.h"

#if 1

static bool bError = false;

static int callbackEvents;

static void CheckResult(ADI_PWR_RESULT result, char *message);
static void setDefaultPower(uint32_t cgu);
// extern void ConfigSoftSwitches(void);

DMC_PARAM DMC_Param_List;

/*
 * Report error if result code is not success
 */
static void CheckResult(ADI_PWR_RESULT result, char *message)
{
	if (result != ADI_PWR_SUCCESS) {
		// printf("%s failed\n", message);
		bError = true;
	}
}

/*
 * Reset processor to default power settings
 */
static void setDefaultPower(uint32_t cgu)
{
	ADI_PWR_RESULT result;

	result = adi_pwr_SetPllControlRegister(cgu, ADI_PWR_PLL_CTL_DISABLE, 0);
	result = adi_pwr_SetPllControlRegister(cgu, ADI_PWR_PLL_CTL_EXIT_ACTIVE, 1);
	result = adi_pwr_SetClkControlRegister(cgu, ADI_PWR_CLK_CTL_DF, DF_DEFAULT);
	result = adi_pwr_SetClkControlRegister(cgu, ADI_PWR_CLK_CTL_MSEL, MSEL_DEFAULT);
	result = adi_pwr_SetClkDivideRegister(cgu, ADI_PWR_CLK_DIV_SYSSEL, SYSSEL_DEFAULT);
	result = adi_pwr_SetClkDivideRegister(cgu, ADI_PWR_CLK_DIV_S0SEL, S0SEL_DEFAULT);
	result = adi_pwr_SetClkDivideRegister(cgu, ADI_PWR_CLK_DIV_S1SEL, S1SEL_DEFAULT);
	result = adi_pwr_SetClkDivideRegister(cgu, ADI_PWR_CLK_DIV_CSEL, CSEL_DEFAULT);
	result = adi_pwr_SetClkDivideRegister(cgu, ADI_PWR_CLK_DIV_DSEL, DSEL_DEFAULT);
}

/*
 * Power event Callback function
 */
static void PWRCallback(void *pCBParam, uint32_t Event, void *pArg)
{
	uint32_t sclk;
	ADI_PWR_MODE  mode;

	switch ( (ADI_PWR_EVENT)Event )
	{
    case ADI_PWR_EVENT_FREQ_PRE_CHANGE:
    	sclk = *(uint32_t*)(pArg);
    	callbackEvents++;
    	break;
    case ADI_PWR_EVENT_FREQ_POST_CHANGE:
    	sclk = *(uint32_t*)(pArg);
    	callbackEvents++;
    	break;
    case ADI_PWR_EVENT_MODE_PRE_CHANGE:
    	mode = (ADI_PWR_MODE)pArg;
    	callbackEvents++;
    	break;
    case ADI_PWR_EVENT_MODE_POST_CHANGE:
    	mode = (ADI_PWR_MODE)pArg;
    	callbackEvents++;
    	break;
    default:
    	break;
	}
}

#endif

void initcode(ADI_ROM_BOOT_CONFIG* pBootStruct)
{
	uint32_t   fcclk = 0u;
	uint32_t   fsysclk = 0u;
	uint32_t   fsclk0 = 0u;
	uint32_t   fsclk1 = 0u;
	uint32_t   fdclk = 0u;

	ADI_PWR_RESULT result;
	ADI_PWR_MODE  mode;
	int expectedEvents = 0;
#if 1
	bError = false;
	callbackEvents = 0;

	//Set the DLLRST bit (bit 11 of the DMC_PHY_CTL0 register) before CGU Initialization. More details at http://www.spd.analog.com/issues/dsp_pte_dev/issuemgr.cgi?item=36-10-0001
	*pREG_DMC0_PHY_CTL0=0x800;

	result = adi_pwr_Init(CGU_DEV, CLKIN);
	CheckResult(result, "adi_pwr_Init");

	result = adi_pwr_InstallCallback(CGU_DEV, PWRCallback);
	CheckResult(result, "adi_pwr_InstallCallback");

#if defined(__ADSPGCC__)
	/* Set CGU clock select register to CLKO2/4 (ARM core) */
	result = adi_pwr_SetClkOutSelectRegister(CGU_DEV, ADI_PWR_CLKOUT_CLKO2);
	CheckResult(result, "adi_pwr_SetClkOutSelectRegister");
#else
	/* Set CGU clock select register to CLKO0/4 (Sharc core) */
	result = adi_pwr_SetClkOutSelectRegister(CGU_DEV, ADI_PWR_CLKOUT_CLKO0);
	CheckResult(result, "adi_pwr_SetClkOutSelectRegister");
#endif

#if defined(CGU_BEFORE_POWER)
	/* Reset processor to default power settings */
	setDefaultPower(CGU_DEV);
#endif
	/* get the power mode */
	result = adi_pwr_GetPowerMode(CGU_DEV, &mode);
	CheckResult(result, "adi_pwr_GetPowerMode");
	if (mode != ADI_PWR_MODE_FULL_ON)
	{
		/* set power mode to full on */
		result = adi_pwr_SetPowerMode(CGU_DEV, ADI_PWR_MODE_FULL_ON);
		CheckResult(result, "adi_pwr_SetPowerMode");
		expectedEvents += 2; /* pre and post event */

		/* get the power mode */
		result = adi_pwr_GetPowerMode(CGU_DEV, &mode);
		CheckResult(result, "adi_pwr_GetPowerMode");
		if (mode != ADI_PWR_MODE_FULL_ON)
		{
			bError = true;
		}
	}

	result = adi_pwr_GetCoreFreq(CGU_DEV, &fcclk);
	CheckResult(result, "adi_pwr_GetCoreFreq");

	result = adi_pwr_GetSystemFreq(CGU_DEV, &fsysclk, &fsclk0, &fsclk1);
	CheckResult(result, "adi_pwr_GetSystemFreq");

	result = adi_pwr_GetDDRClkFreq(CGU_DEV, &fdclk);
	CheckResult(result, "adi_pwr_GetDDRClkFreq");

	//printf("CCLK = %d, SYSCLK = %d, SCLK0 = %d, SCLK1 = %d, DCLK = %d\n", fcclk, fsysclk, fsclk0, fsclk1, fdclk);


	result = adi_pwr_RemoveCallback(CGU_DEV, PWRCallback);
	CheckResult(result, "adi_pwr_RemoveCallback");

	/* check num callback events */
	if (expectedEvents != callbackEvents) {
		bError = true;
		//printf("Wrong number of callback events\n");
	}
#if !defined(CGU_BEFORE_POWER)
	/* Reset processor to default power settings */
	setDefaultPower(CGU_DEV);
#endif


	//Clear the DLLRST bit (bit 11 of the DMC_PHY_CTL0 register) after CGU Initialization. More details at http://www.spd.analog.com/issues/dsp_pte_dev/issuemgr.cgi?item=36-10-0001
	*pREG_DMC0_PHY_CTL0=0x000;
#else
	Init_CGU();
#endif
	int i = 0;

	//Initialize the DMC parameters list
	DMC_Param_List.dmc_no=0;
	DMC_Param_List.ddr_mode=DDR2_MODE;
	DMC_Param_List.phy_init_required=true;
	DMC_Param_List.cclk_dclk_ratio=1;
	DMC_Param_List.anomaly_20000037_applicable=true;
	DMC_Param_List.dmc_ctl_value=DMC_CTL_VALUE;
	DMC_Param_List.dmc_cfg_value=DMC_CFG_VALUE;
	DMC_Param_List.dmc_tr0_value=DMC_TR0_VALUE;
	DMC_Param_List.dmc_tr1_value=DMC_TR1_VALUE;
	DMC_Param_List.dmc_tr2_value=DMC_TR2_VALUE;
	DMC_Param_List.dmc_mr0_value=DMC_MR0_VALUE;
	DMC_Param_List.dmc_mr1_value=DMC_MR1_VALUE;
	DMC_Param_List.dmc_mr2_value=DMC_MR2_VALUE;
	DMC_Param_List.padctl2_value=DMC_PADCTL2_VALUE;
	DMC_Param_List.dmc_cphyctl_value=DMC_CPHYCTL_VALUE;
	DMC_Param_List.dmc_data_calib_add_value=DMC0_DATA_CALIB_ADD;

	//Initialize DMC0 now
	DMCInit(DMC_Param_List);


	return;
}


#endif /* __ADSPSC584__ */
